import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Button, Divider, Sheet, Typography } from "@mui/joy";
import { useTranslation } from "react-i18next";
import { Link } from "react-router-dom";
/**
 * @component
 * @param {NerdAppBarProps} props NerdAppBar props
 * @param {string} props.title title of the app
 * @param {string} props.logo Logo of the app
 * @param {string} props.variant Size of the appBar (dense or regular)
 * @param {Array<NerdAppBarContent | React.ReactElement>} props.content Height of the appbar. Could be dense or regular.
 * @returns {React.ReactElement | null} UserDisplay component
 */
export default function NerdAppBar({ title, logo, content = [], variant = "dense", appRoot = "/", divider = true, }) {
    const { t } = useTranslation();
    return (_jsx("header", { style: {
            zIndex: 1,
            position: "fixed",
            top: 0,
            left: "auto",
            right: 0,
            display: "flex",
            width: "100%",
            flexShrink: 0,
            boxSizing: "border-box",
            flexDirection: "column",
        }, children: _jsxs(Sheet, { sx: {
                display: "flex",
                alignItems: "center",
                px: 2,
                minHeight: 48,
            }, children: [_jsx(Button, { size: "sm", component: Link, to: appRoot, variant: "plain", color: "neutral", startDecorator: logo ? (_jsx("img", { alt: t("global.logo"), src: logo, height: variant === "dense" ? "30px" : "45px" })) : undefined, sx: {
                        textDecoration: "none !important",
                    }, children: _jsx(Typography, { sx: {
                            display: { xs: "none", md: "inherit" },
                            ml: "6px",
                            fontFamily: "futura-pt-condensed, sans-serif;",
                            fontSize: 24,
                        }, children: title?.toUpperCase() }) }), content, divider ? _jsx(Divider, {}) : []] }) }));
}
