import { jsx as _jsx } from "react/jsx-runtime";
import { createElement as _createElement } from "react";
import { Box, Input } from "@mui/joy";
import { createRef, useEffect, useState, } from "react";
/**
 * NerdCodeField: A component that displays multiple inputs for a one-time-code.
 *
 * @param {{length, onChange, value, TextfieldProps}} params List of params
 * @returns {React.ReactElement} Returns the CodeField component
 */
export default function NerdCodeField({ length = 6, onChange, value = "", inputProps, }) {
    const arrLength = length;
    const [elRefs, setElRefs] = useState([]);
    useEffect(() => {
        setElRefs((elRefs) => Array(arrLength)
            .fill(0)
            .map((_, i) => elRefs[i] || createRef()));
    }, [arrLength]);
    const handleOnPaste = (e) => {
        const pastedValue = e.clipboardData.getData("Text");
        e.stopPropagation();
        e.preventDefault();
        if (/^\d+$/.test(pastedValue)) {
            moveTo(length - 1);
            return onChange(pastedValue.substring(0, length));
        }
    };
    const handleChange = (index) => (e) => {
        const digit = e.target.value.trim(); // single digit keyboard input case
        if (digit === value[index])
            return;
        if (/^\d$/.test(digit)) {
            onChange(value.substring(0, index) + digit + value.substring(index + 1));
            moveToNext(index);
        }
        else if (digit === "") {
            onChange(value.substring(0, index) + " " + value.substring(index + 1));
        }
    };
    const moveTo = (index) => {
        elRefs[index].current?.focus();
    };
    const moveToNext = (index) => {
        if (index < length - 1)
            elRefs[index + 1].current?.focus();
    };
    const moveToPrevious = (index) => {
        if (index > 0)
            elRefs[index - 1].current?.focus();
    };
    const handleKeyUp = (t) => (e) => {
        if (e.code == "ArrowRight") {
            // keyboard right arrow
            moveToNext(t);
        }
        else if (e.code == "ArrowLeft") {
            // keyboard left arrow
            moveToPrevious(t);
        }
        else if (e.code == "Backspace") {
            // delete
            moveToPrevious(t);
        }
        else if (/^\d$/.test(e.key) && e.key === value[t]) {
            moveToNext(t);
        }
    };
    return (_jsx(Box, { sx: { display: "flex", gap: 1 }, children: [...Array(length).keys()].map((t) => (_createElement(Input, { placeholder: "X", ...inputProps, key: t, autoFocus: value.length === 0 ? t === 0 : t === value.length - 1, onFocus: (event) => {
                event.target.select();
            }, onKeyUp: handleKeyUp(t), slotProps: {
                input: {
                    ref: elRefs[t],
                    inputtype: "numeric",
                    onPaste: t === 0 ? handleOnPaste : undefined,
                    pattern: "[0–9]*",
                    maxLength: 1,
                    autoComplete: t === 0 ? "one-time-code" : "off",
                    style: {
                        textAlign: "center",
                    },
                },
            }, value: value.length >= t && value[t] !== " " && value[t] !== undefined
                ? value[t]
                : "", onChange: handleChange(t) }))) }));
}
