import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { ClickAwayListener } from "@mui/base";
import { LocalizationProvider, StaticDateRangePicker, } from "@mui/x-date-pickers-pro";
import { DateTime } from "luxon";
import { useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import i18n from "../i18n";
import { NerdPickerWrapper } from "../nerdPickersWrapper";
import { CustomAdapterLuxon } from "../nerdDatePicker/customAdapterLuxon";
import { Box, Typography, Grid, Input, FormControl, FormHelperText, useTheme, } from "@mui/joy";
const defaultlocaleStringFormat = {
    weekday: "long",
    month: "long",
    day: "2-digit",
};
/**
 *
 * @param {NerdDateRangePickerProps} props Props
 * @returns {ReactElement} NerdDateTimePicker
 */
export default function NerdDateRangePicker(props) {
    const { startInputProps, endInputProps, dateRangePickerProps, onChange, value, hideIcon, localeStringFormat = defaultlocaleStringFormat, } = props;
    const theme = useTheme();
    const [startTextFieldValue, setStartTextFieldValue] = useState(value !== null && value[0] !== null
        ? value[0].toLocaleString(localeStringFormat)
        : "");
    const [endTextFieldValue, setEndTextFieldValue] = useState(value !== null && value[1] !== null
        ? value[1].toLocaleString(localeStringFormat)
        : "");
    const [tmpValue, setTmpValue] = useState(value);
    const [anchorEl, setAnchorEl] = useState(null);
    const [startError, setStartError] = useState(false);
    const [endError, setEndError] = useState(false);
    const adapter = new CustomAdapterLuxon({ locale: i18n.language });
    const { t } = useTranslation("components", { i18n: i18n });
    const handleClick = (event) => {
        setAnchorEl(event.currentTarget);
    };
    const handleClose = () => {
        setAnchorEl(null);
    };
    const setTextFieldsValue = (value) => {
        let startDate = "";
        let endDate = "";
        if (value[0] !== null)
            startDate = value[0].toLocaleString(localeStringFormat);
        if (value[1] !== null)
            endDate = value[1].toLocaleString(localeStringFormat);
        setStartTextFieldValue(startDate);
        setEndTextFieldValue(endDate);
    };
    useEffect(() => {
        setTmpValue(value);
        setTextFieldsValue(value);
    }, [value]);
    useEffect(() => {
        setTextFieldsValue(tmpValue);
    }, [tmpValue]);
    const onStartTextFieldValueChange = (newValue) => {
        setStartTextFieldValue(newValue);
    };
    const onEndTextFieldValueChange = (newValue) => {
        setEndTextFieldValue(newValue);
    };
    const onStartTextFieldBlur = (e) => {
        const date = adapter.date(e.target.value);
        if (date?.isValid) {
            const newDate = value[0]?.set({
                day: date.day,
                month: date.month,
                ...(e.target.value.includes(date.year.toString()) && {
                    year: date.year,
                }),
            });
            if (value[0] &&
                value[1] &&
                newDate &&
                newDate.toMillis() != value[0].toMillis()) {
                setStartTextFieldValue(date.toLocaleString(localeStringFormat));
                onChange([newDate, newDate.plus(value[1].diff(value[0]))]);
            }
        }
    };
    const onEndTextFieldBlur = (e) => {
        const date = adapter.date(e.target.value);
        if (date?.isValid) {
            const newDate = value[1]?.set({
                day: date.day,
                month: date.month,
                ...(e.target.value.includes(date.year.toString()) && {
                    year: date.year,
                }),
            });
            if (value[1] && newDate && newDate.toMillis() != value[1].toMillis()) {
                setStartTextFieldValue(date.toLocaleString(localeStringFormat));
                onChange([value[0], newDate]);
            }
        }
    };
    const onDateChange = (newValue) => {
        setTmpValue(newValue);
        onChange(newValue);
    };
    useEffect(() => {
        let startError = false;
        let endError = false;
        if ((startInputProps && !adapter.date(startTextFieldValue)?.isValid) ||
            startInputProps?.error)
            startError = true;
        if ((endInputProps && !adapter.date(endTextFieldValue)?.isValid) ||
            endInputProps?.error)
            endError = true;
        setEndError(endError);
        setStartError(startError);
    }, [startTextFieldValue, endTextFieldValue]);
    return (_jsx(ClickAwayListener, { onClickAway: handleClose, children: _jsx("div", { children: _jsxs(LocalizationProvider, { dateAdapter: CustomAdapterLuxon, adapterLocale: "en-US", children: [_jsxs(Box, { sx: { display: "flex", alignItems: "baseline" }, children: [!hideIcon && (_jsx(Typography, { sx: { color: "action.active", mr: 1, my: 0.5 }, children: _jsx("i", { className: "fa-solid fa-calendar-range" }) })), _jsxs(Grid, { container: true, spacing: 1, children: [_jsx(Grid, { children: _jsxs(FormControl, { children: [_jsx(Input, { size: "sm", ...startInputProps, error: startError, value: startTextFieldValue, onClick: (e) => {
                                                        handleClick(e);
                                                        startInputProps?.onClick?.(e);
                                                    }, onChange: (e) => {
                                                        onStartTextFieldValueChange(e.target.value);
                                                        startInputProps?.onChange?.(e);
                                                    }, onBlur: (e) => {
                                                        onStartTextFieldBlur(e);
                                                        startInputProps?.onBlur?.(e);
                                                    } }), _jsx(FormHelperText, { children: !anchorEl
                                                        ? startError
                                                            ? t("nerdDatePicker.invalid_date")
                                                            : ""
                                                        : "" })] }) }), _jsx(Grid, { children: _jsx(Typography, { fontSize: 24, children: "-" }) }), _jsx(Grid, { children: _jsxs(FormControl, { children: [_jsx(Input, { size: "sm", ...endInputProps, error: endError, value: endTextFieldValue, onClick: (e) => {
                                                        handleClick(e);
                                                        endInputProps?.onClick?.(e);
                                                    }, onChange: (e) => {
                                                        onEndTextFieldValueChange(e.target.value);
                                                        endInputProps?.onChange?.(e);
                                                    }, onBlur: (e) => {
                                                        onEndTextFieldBlur(e);
                                                        endInputProps?.onBlur?.(e);
                                                    } }), _jsx(FormHelperText, { children: !anchorEl
                                                        ? endError
                                                            ? t("nerdDatePicker.invalid_date")
                                                            : ""
                                                        : "" })] }) })] })] }), _jsx(NerdPickerWrapper, { popperProps: {
                            open: anchorEl === null ? false : true,
                            anchorEl: anchorEl,
                            style: {
                                zIndex: theme.zIndex.modal,
                                boxShadow: theme.shadow.md,
                                borderRadius: "4px",
                                backgroundColor: "inherit",
                            },
                        }, modalProps: {
                            open: anchorEl === null ? false : true,
                            onClose: handleClose,
                        }, dialogActionBarProps: {
                            onAccept() {
                                onChange(tmpValue);
                                handleClose();
                            },
                            onCancel() {
                                setTmpValue(value);
                                handleClose();
                            },
                            onClear() {
                                onChange([null, null]);
                                handleClose();
                            },
                            onSetToday() {
                                onChange([DateTime.now(), DateTime.now()]);
                                handleClose();
                            },
                            actions: dateRangePickerProps?.componentsProps?.actionBar
                                ?.actions || ["today", "cancel", "accept"],
                        }, popperActionBarProps: {
                            onAccept() {
                                onChange(tmpValue);
                                handleClose();
                            },
                            onCancel() {
                                setTmpValue(value);
                                handleClose();
                            },
                            onClear() {
                                onChange([null, null]);
                                handleClose();
                            },
                            onSetToday() {
                                onChange([DateTime.now(), DateTime.now()]);
                                handleClose();
                            },
                            actions: dateRangePickerProps?.componentsProps?.actionBar
                                ?.actions || ["today"],
                        }, children: _jsx(Box, { sx: {
                                "& .MuiPickerStaticWrapper-content": {
                                    borderRadius: "4px",
                                },
                            }, children: _jsx(StaticDateRangePicker, { showToolbar: false, componentsProps: {
                                    actionBar: {
                                        actions: [],
                                    },
                                }, disableMaskedInput: true, ...dateRangePickerProps, value: tmpValue, onAccept: handleClose, onChange: (value, keyboardInputValue) => {
                                    onDateChange(value);
                                    dateRangePickerProps?.onChange?.(value, keyboardInputValue);
                                }, renderInput: () => _jsx(_Fragment, {}) }) }) })] }) }) }));
}
