import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { useCallback, useContext, useEffect, useMemo, useRef, } from "react";
import { NerdFileUploadContext } from "./nerdFileUploadWrapper";
import { nanoid } from "@reduxjs/toolkit";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../i18n";
import { Button } from "@mui/joy";
/**
 *
 * @param {NerdFileUploadProps} props Component props
 * @returns {ReactElement} NerdFileUpload
 */
export default function NerdFileUpload(props) {
    const context = useContext(NerdFileUploadContext);
    const id = useMemo(() => nanoid(), []);
    const { dropZoneLabel, multiple, accept, maxSize, uploadFile, onError, slotProps, slots, } = props;
    const { t } = useTranslation("components", { i18n: nerdUII18n });
    useEffect(() => {
        context.subscribe(id, handleDrop, dropZoneLabel);
        return () => {
            context.unsubscribe(id);
        };
    }, []);
    const fileInput = useRef(null);
    const checkFile = useCallback((file) => {
        try {
            if (accept && !accept.includes(file.type)) {
                throw new Error("The file type is not allowed.");
            }
            else if (maxSize && file.size > maxSize) {
                throw new Error(`File size exceeds limit (${maxSize / 1000000} MB)`);
            }
            else {
                return true;
            }
        }
        catch (error) {
            if (error instanceof Error)
                onError?.(error);
            return;
        }
    }, [maxSize, accept]);
    /**
     *
     * @param {ChangeEvent<HTMLInputElement>} e ChangeEvent
     */
    function handleChange(e) {
        const newFiles = e.target.files;
        const files = [];
        if (newFiles != null) {
            for (let i = 0; i < newFiles.length; i++) {
                const file = newFiles[i];
                const check = checkFile(file);
                if (check === true)
                    files.push(file);
            }
            if (multiple)
                uploadFile?.(files);
            else
                uploadFile?.(files[0]);
        }
        e.target.value = "";
    }
    /**
     *
     * @param {DragEvent<HTMLDivElement>} e Drag event
     */
    function handleDrop(e) {
        const limit = multiple ? e.dataTransfer.files.length : 1;
        const files = [];
        for (let i = 0; i < limit; i++) {
            const file = e.dataTransfer.files[i];
            const check = checkFile(file);
            if (check === true)
                files.push(file);
        }
        if (multiple)
            uploadFile?.(files);
        else
            uploadFile?.(files[0]);
    }
    return (_jsxs(_Fragment, { children: [_jsx("input", { type: "file", ref: fileInput, multiple: multiple, ...(accept ? { accept: accept.join(",") } : {}), style: {
                    clip: "rect(0 0 0 0)",
                    clipPath: "inset(50%)",
                    height: 1,
                    position: "absolute",
                    bottom: 0,
                    left: 0,
                    whiteSpace: "nowrap",
                    width: 1,
                }, onChange: handleChange }), slots?.root ? (_jsx(slots.root, { ...slotProps?.root, 
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                onClick: (e) => {
                    fileInput.current?.click();
                    slotProps?.root?.onClick?.(e);
                } })) : (_jsx(Button, { variant: "outlined", startDecorator: _jsx("i", { className: "fa-solid fa-cloud-arrow-up" }), ...slotProps?.root, onClick: () => {
                    fileInput.current?.click();
                }, children: slotProps?.root?.children || t("nerdFileUpload.browse_files") }))] }));
}
