import { jsx as _jsx } from "react/jsx-runtime";
import { createRoot } from "react-dom/client";
import { unified } from "unified";
import remarkDirective from "remark-directive";
import remarkParse from "remark-parse";
import remarkGfm from "remark-gfm";
import remarkHtml from "remark-html";
import { ReactMarkdown } from "react-markdown/lib/react-markdown";
import { Button } from "@mui/joy";
import remarkDirectiveRehype from "./remarkDirectiveRehype";
import { CssVarsProvider } from "@mui/joy";
import { WidgetType } from "@codemirror/view";
export class BulletListWidget extends WidgetType {
    id;
    text;
    nested;
    level;
    offset;
    constructor(id, text, nested, level, offset) {
        super();
        this.id = id;
        this.text = text;
        this.nested = nested;
        this.level = level;
        this.offset = offset;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM() {
        const parent = document.createElement("span");
        parent.innerHTML = this.level % 2 == 0 ? "&#x25E6;" : "&#x2022;";
        if (this.nested) {
            parent.style.marginLeft = `${16 * this.level + this.offset / 2}px`;
        }
        else {
            parent.style.marginLeft = "16px";
        }
        return parent;
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
export class TaskWidget extends WidgetType {
    id;
    text;
    from;
    to;
    constructor(id, text, from, to) {
        super();
        this.id = id;
        this.text = text;
        this.from = from;
        this.to = to;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM(view) {
        const parent = document.createElement("span");
        const checkbox = document.createElement("input");
        checkbox.type = "checkbox";
        if (this.text.includes("x") || this.text.includes("X")) {
            checkbox.checked = true;
        }
        checkbox.addEventListener("mousedown", (e) => {
            e.preventDefault();
            e.stopPropagation();
            view.dispatch(view.state.changeByRange((range) => ({
                changes: [
                    {
                        from: this.from + 1,
                        to: this.from + 2,
                        insert: checkbox.checked ? " " : "x",
                    },
                ],
                range: range,
            })));
        });
        parent.append(checkbox);
        return parent;
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
export class MarkdownWidget extends WidgetType {
    id;
    text;
    constructor(id, text) {
        super();
        this.id = id;
        this.text = text;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM() {
        const parent = document.createElement("span");
        const processor = unified().use(remarkParse).use(remarkGfm).use(remarkHtml);
        parent.innerHTML = String(processor.processSync(this.text));
        const child = parent.firstChild
            ? parent.firstChild
            : parent.firstChild;
        if (child) {
            child.style.margin = "0";
            child.style.display = "inline-block";
            if (child.tagName == "UL" && this.text.match(/- \[( |x|X)\]/))
                child.style.listStyleType = "none";
        }
        return parent;
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
export class MarkdownTextDirectiveWidget extends WidgetType {
    id;
    text;
    constructor(id, text) {
        super();
        this.id = id;
        this.text = text;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM() {
        try {
            const parent = document.createElement("span");
            const el = document.createElement("span");
            parent.appendChild(el);
            parent.style.display = "inline-block";
            el.style.display = "inline-block";
            const root = createRoot(el);
            root.render(_jsx(CssVarsProvider, { children: _jsx(ReactMarkdown, { remarkPlugins: [remarkGfm, remarkDirective, remarkDirectiveRehype], components: { "button-component": Button }, children: this.text }) }));
            return parent.appendChild(el);
        }
        catch (e) {
            console.error(e);
            return document.createElement("div");
        }
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
