import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React from "react";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../../i18n";
import { useSelector } from "react-redux";
import { nerdNotifierNotificationSelector } from "../../redux";
import { Box, Button, DialogActions, DialogContent, DialogTitle, LinearProgress, Modal, ModalDialog, Stack, Typography, } from "@mui/joy";
/**
 *
 * @param {NerdNotifierProps & {open: boolean} } param0 Component props
 * @returns {JSX.Element} Intrusive nerdNotifier
 */
export function Intrusive({ actionPrimaryProps = {
    variant: "plain",
    color: "primary",
}, actionSecondaryProps = {
    variant: "plain",
    color: "neutral",
}, open, handleClose, }) {
    const { autoHideDuration = 5000, autohide = true, body, icon, severity = "primary", showIcon = true, subtitle, title, } = useSelector(nerdNotifierNotificationSelector) || {};
    const [progress, setProgress] = React.useState(0);
    const timer = React.useRef(0);
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    React.useEffect(() => {
        timer.current = window.setInterval(() => {
            setProgress((prevProgress) => {
                return prevProgress >= 100
                    ? 0
                    : prevProgress + 100 / (autoHideDuration / 10);
            });
        }, 10);
        return () => {
            clearInterval(timer.current);
        };
    }, []);
    React.useEffect(() => {
        if (progress >= 100 && autohide) {
            handleClose();
        }
    }, [progress, autohide, open]);
    const severityIcon = severityIcons(severity);
    return (_jsx(Modal, { open: open, onClose: handleClose, keepMounted: true, children: _jsx(ModalDialog, { children: _jsxs(Stack, { spacing: 1, children: [autohide && (_jsx(LinearProgress, { determinate: true, value: progress, color: severity })), showIcon && (_jsx(Box, { sx: {
                            display: "flex",
                            marginTop: "1rem",
                            justifyContent: "center",
                        }, children: icon != undefined ? icon : severityIcon })), title && _jsx(DialogTitle, { id: "alert-dialog-title", children: title }), _jsxs(DialogContent, { children: [subtitle && (_jsx(Typography, { level: "body-sm", id: "alert-dialog-description", children: subtitle })), body] }), !autohide && (_jsxs(DialogActions, { children: [_jsx(Button, { onClick: handleClose, ...actionPrimaryProps, children: t("global.ok") }), _jsx(Button, { onClick: handleClose, ...actionSecondaryProps, children: t("global.cancel") })] }))] }) }) }));
}
/**
 *
 * @param {string} severityText Severity text
 * @returns {ReactElement | null} Severity Icon
 */
function severityIcons(severityText) {
    switch (severityText) {
        case "error":
            return (_jsx(Typography, { color: "danger", children: _jsx("i", { className: "fa-solid fa-circle-exclamation" }) }));
        case "info":
            return (_jsx(Typography, { color: "primary", children: _jsx("i", { className: "fa-solid fa-circle-info" }) }));
        case "success":
            return (_jsx(Typography, { color: "success", children: _jsx("i", { className: "fa-solid fa-circle-check" }) }));
        case "warning":
            return (_jsx(Typography, { color: "warning", children: _jsx("i", { className: "fa-solid fa-triangle-exclamation" }) }));
        default:
            return null;
    }
}
