import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../../i18n";
import { Alert, Box, Button, IconButton, LinearProgress, Snackbar, Stack, Typography, } from "@mui/joy";
import { useDispatch } from "react-redux";
import { setOpenFeedbackDialog } from "../../redux/nerdFeedback";
import { useSelector } from "react-redux";
import { nerdNotifierNotificationSelector } from "../../redux";
import { severityIcon } from "../const";
import * as Sentry from "@sentry/react";
import { useEffect, useRef, useState } from "react";
/**
 *
 * @param {NerdNotificationProps & openHookAction} param0 components props
 * @returns {JSX.Element} NonIntrusive nerdNotifier
 */
export function NonIntrusive({ handleClose, open, hideReportButton, }) {
    const { title, severity = "primary", subtitle, body, position = {
        vertical: "bottom",
        horizontal: "right",
    }, autoHideDuration = 5000, autohide, error, } = useSelector(nerdNotifierNotificationSelector) || {};
    const [progress, setProgress] = useState(0);
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    const timer = useRef(0);
    const dispatch = useDispatch();
    useEffect(() => {
        timer.current = window.setInterval(() => {
            setProgress((prevProgress) => {
                return prevProgress >= 100
                    ? 0
                    : prevProgress + 100 / (autoHideDuration / 10);
            });
        }, 10);
        return () => {
            clearInterval(timer.current);
        };
    }, []);
    useEffect(() => {
        if (progress >= 100 && autohide) {
            clearInterval(timer.current);
            handleClose();
        }
    }, [progress, autohide]);
    return (_jsx(Box, { children: _jsx(Snackbar, { open: open, autoHideDuration: autohide ? autoHideDuration : null, anchorOrigin: position, onClose: (_event, reason) => {
                if (reason === "clickaway") {
                    return;
                }
            }, sx: {
                p: 0,
                border: "none",
                backgroundColor: "none",
            }, children: _jsxs(Stack, { spacing: 1, children: [autohide && (_jsx(LinearProgress, { determinate: true, value: progress, color: severity })), _jsx(Alert, { variant: "solid", color: severity, sx: {
                            width: "100%",
                            alignItems: "center",
                        }, startDecorator: severityIcon[severity], endDecorator: _jsx(IconButton, { onClick: handleClose, sx: {
                                ml: 1,
                            }, variant: "solid", size: "sm", color: severity, children: _jsx("i", { className: "fa-solid fa-xmark" }) }), children: _jsxs("div", { children: [title && (_jsx(Typography, { sx: { color: "inherit" }, children: title })), subtitle && (_jsx(Typography, { sx: { color: "inherit" }, level: "body-sm", children: subtitle })), body && (_jsx(Typography, { sx: { color: "inherit" }, level: "body-xs", children: body }))] }) }), Sentry.getClient()?.getDsn()?.publicKey &&
                        (severity === "danger" || severity === "warning") &&
                        !hideReportButton ? (_jsx(Button, { size: "sm", onClick: () => {
                            dispatch(setOpenFeedbackDialog({ open: true, error }));
                            handleClose();
                        }, startDecorator: _jsx("i", { className: "fa-regular fa-comment-exclamation" }), color: "neutral", children: t("components:nerdNotifier.report_issue") })) : null] }) }) }));
}
