import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, Divider, IconButton, Sheet } from "@mui/joy";
import { Popper, ClickAwayListener } from "@mui/base";
import { Fragment, useCallback, useEffect, useRef, useState, } from "react";
const offset = 80;
/**
 *
 * @param {NerdToolbarProps} props props
 * @returns {ReactElement} NerdToolbar
 */
export function NerdToolbar(props) {
    const { groups, rightPinnedElements = [] } = props;
    const containerRef = useRef(null);
    const pinnedItemsRef = useRef(null);
    const groupsRef = useRef(null);
    const [indexMeta, setIndexMeta] = useState({
        index: -1,
        width: 0,
        totalWidth: 0,
    });
    const [anchorEl, setAnchorEl] = useState(null);
    const handleClick = (event) => {
        setAnchorEl(event.currentTarget);
    };
    const handleClose = () => {
        setAnchorEl(null);
    };
    const open = Boolean(anchorEl);
    const handleResize = useCallback(() => {
        const containerWidth = containerRef.current?.offsetWidth || 0;
        const pinnedItemsWidth = pinnedItemsRef.current?.offsetWidth || 0;
        let totalWidth = 0;
        if (containerWidth !== 0 && groupsRef.current) {
            const items = groupsRef.current.querySelectorAll(":scope > div");
            if (items.length > 0) {
                for (let i = 0; i < items.length; i++) {
                    const node = items[i];
                    const currentWidth = node.scrollWidth;
                    totalWidth += currentWidth;
                    if (totalWidth > containerWidth - pinnedItemsWidth - offset) {
                        setIndexMeta({
                            index: i,
                            width: currentWidth,
                            totalWidth,
                        });
                        break;
                    }
                    else {
                        setIndexMeta((indexMeta) => {
                            if (groupsRef.current &&
                                indexMeta.index < groups.length &&
                                indexMeta.totalWidth <
                                    containerWidth - pinnedItemsWidth - offset) {
                                if (indexMeta.index + 1 === groups.length)
                                    handleClose();
                                return {
                                    ...indexMeta,
                                    index: indexMeta.index < 0 ? 1 : indexMeta.index + 1,
                                };
                            }
                            else {
                                return indexMeta;
                            }
                        });
                    }
                }
            }
            else {
                setIndexMeta((indexMeta) => {
                    if (groupsRef.current &&
                        indexMeta.index < groups.length &&
                        indexMeta.totalWidth < containerWidth - pinnedItemsWidth - offset) {
                        return {
                            ...indexMeta,
                            index: 1,
                        };
                    }
                    else {
                        return indexMeta;
                    }
                });
            }
        }
    }, [groupsRef, containerRef, pinnedItemsRef, indexMeta, groups]);
    useEffect(() => {
        window.addEventListener("resize", handleResize);
        return () => {
            window.removeEventListener("resize", handleResize);
        };
    }, [containerRef, pinnedItemsRef, groupsRef]);
    useEffect(() => {
        const containerWidth = containerRef.current?.offsetWidth || 0;
        const groupsWidth = groupsRef.current?.offsetWidth || 0;
        const pinnedItemsWidth = pinnedItemsRef.current?.offsetWidth || 0;
        if (groupsWidth > containerWidth - pinnedItemsWidth - offset &&
            groupsRef.current) {
            const items = groupsRef.current.querySelectorAll(":scope > div");
            let totalWidth = 0;
            if (items.length > 0) {
                for (let i = items.length - 1; i >= 0; i--) {
                    const node = items[i];
                    const currentWidth = node.scrollWidth;
                    totalWidth += currentWidth;
                    if (groupsWidth - totalWidth <=
                        containerWidth - pinnedItemsWidth - offset) {
                        setIndexMeta({
                            index: i,
                            width: currentWidth,
                            totalWidth: groupsWidth - totalWidth,
                        });
                        break;
                    }
                }
            }
        }
    }, [containerRef, groupsRef, pinnedItemsRef]);
    return (_jsx(Sheet, { variant: "soft", sx: {
            borderRadius: "xl",
            px: 2,
            py: 0.5,
            overflow: "hidden",
            minWidth: 74 + (pinnedItemsRef.current?.offsetWidth || 0),
        }, children: _jsxs(Box, { ref: containerRef, width: "100%", display: "flex", gap: 0.5, children: [_jsx(Box, { ref: groupsRef, display: "flex", alignItems: "center", gap: 0.5, position: "relative", children: groups.map((g, i) => {
                        if (i < indexMeta.index || indexMeta.index === -1)
                            return (_jsxs(Box, { display: "flex", gap: 0.5, sx: {
                                    opacity: 0,
                                    animationTimingFunction: "ease-out",
                                    animationName: "fade",
                                    animationDelay: "50ms",
                                    animationDuration: "150ms",
                                    animationFillMode: "forwards",
                                    "@keyframes fade": {
                                        from: { opacity: 0 },
                                        to: { opacity: 1 },
                                    },
                                }, children: [g, i < groups.length - 1 ? (_jsx(Divider, { orientation: "vertical" })) : null] }, i));
                        else
                            return null;
                    }) }), _jsx(IconButton, { size: "sm", id: "subMenu", onClick: handleClick, variant: "soft", color: "neutral", sx: {
                        visibility: indexMeta.index !== -1 && indexMeta.index < groups.length
                            ? "visible"
                            : "hidden",
                    }, children: _jsx("i", { className: "fa-solid fa-ellipsis-vertical" }) }), _jsx(Popper, { open: open, anchorEl: anchorEl, children: _jsx(ClickAwayListener, { onClickAway: handleClose, children: _jsx(Sheet, { variant: "soft", sx: {
                                p: 0.5,
                                width: "100%",
                                overflow: "auto",
                                boxShadow: (theme) => theme.shadow.md,
                            }, children: _jsx(Box, { width: "100%", children: _jsx(Box, { display: "flex", alignItems: "center", gap: 0.5, position: "relative", width: "fit-content", flexWrap: "wrap", children: groups.map((g, i) => {
                                        if (i >= indexMeta.index)
                                            return (_jsxs(Fragment, { children: [g, i < groups.length - 1 ? (_jsx(Divider, { orientation: "vertical" })) : null] }, i));
                                        else
                                            return null;
                                    }) }) }) }) }) }), _jsx(Box, { ref: pinnedItemsRef, sx: {
                        position: "absolute",
                        top: 0,
                        bottom: 0,
                        margin: "auto 0",
                        right: 16,
                    }, display: "flex", alignItems: "center", gap: 0.5, children: rightPinnedElements.map((g, i) => (_jsxs(Fragment, { children: [g, i < rightPinnedElements.length - 1 ? (_jsx(Divider, { orientation: "vertical" })) : null] }, i))) })] }) }));
}
